;*******************************************
;*Simple & effective rs232 circular buffer *
;*  Interupt driven, thus 'transparent' to *
;*  User functions                         *
;*******************************************
;* Origin: 5/99 by Jack Tidwell
;*
;* Modified 2000.08.16. by VFX.
;**************************************************************************



.equ	BUFSIZE = 10

.def	rxhead	=r16
.def	rxtail	=r17
.def	tempchar=r18

.def	txhead	=r16
.def	txtail	=r17


.DSEG 
URxBuf:	.BYTE BUFSIZE		;UART Receive  Buffer
UTxBuf: .BYTE BUFSIZE		;UART Transmit Buffer



.CSEG
;*************************************************************************
;*  Text Const
;*
Copyr:	.DB	"(C) 2000, PLC1 V1.0", 10,13,"PLC szimulator by VFX",10,13
Ready:	.DB	"Ready>>",0
Hiba:	.DB	"Error..",10,13,0

;************************************************************************
;** Init_UART
;**
;************************************************************************
Init_UART:
	LDI	R16,low(2*Copyr)
	STD	Y+TXT+0-RAMSTRT,R16
	LDI	R16,high(2*Copyr)
	STD	Y+TXT+1-RAMSTRT,R16		;UART init TEXT ADDR

	LDI	R16,low(SendTXT)
	STD	Y+UTxAddr+0-RAMSTRT,R16
	LDI	R16,high(SendTXT)
	STD	Y+UTxAddr+1-RAMSTRT,R16 ;UART TX INT. ADDR

	LDI	R16,BaudRate
	OUT	UBRR,R16		;BaudRate Gen.
	LDI	R16,128+64+16+8		;RXCIE,TXCIE,RXEN,TXEN
	OUT	UCR,R16

	ldi	R16,0
	STD	Y+prxtail-RAMSTRT,R16
	STD	Y+prxhead-RAMSTRT,R16
	STD	Y+ptxtail-RAMSTRT,R16
	STD	Y+ptxhead-RAMSTRT,R16
	ret

;*******************************************
;* UART RX COMPLET 
;*******************************************
UART_RXC:
	push	zl
	push	zh
	push	rxtail
	in	zl,SREG			; preserve main OS status reg.
	push	zl
	push	r2
	clr	r2
	ldi	zl,low(URxBuf)		; ptr to our rxbuffer... z = Buf[rxtail]
	ldi	zh,high(URxBuf)
	ldd	rxtail,Y+prxtail-RAMSTRT
	add	zl,rxtail	
	adc	zh,r2
	in	r2,UDR			; get the incomming char
	st	z,r2			; Buf[rxtail] = UDR
	inc	rxtail
	cpi	rxtail,BUFSIZE
	brlo	Rs232inx
	clr	rxtail			; Circle around to the first buffer pos.
Rs232inx:
	std	Y+prxtail-RAMSTRT,rxtail

		;***** Xoff calcs go here *****
	pop	r2
	pop	zl
	out	SREG,zl			; restore previous status reg
	pop	rxtail
	pop	zh
	pop	zl
	reti				; return to normal pgm status.


;*********************************
UART_DRE:	RETI


;
;************************************************
;* The RS232 RxD FIFO manager			*
;* C=1, tempchar (R18)=RS232 rx-ed char		*
;************************************************
Rs232GetByte:
	clr	tempchar
	ldd	rxtail,Y+prxtail-RAMSTRT
	ldd	rxhead,Y+prxhead-RAMSTRT
	cp	rxhead,rxtail
	breq	Rs232GetBytexit
	ldi	zl,low(URxBuf)		; Pointer to the rxbuffer (FIFO)
	ldi	zh,high(URxBuf)
	add	zl,rxhead
	adc	zh,tempchar		; add address offset...  z = &Buf[rxhead]
	ld	tempchar,z		; tempchar = Buf[rxhead++]
	inc	rxhead
	cpi	rxhead,BUFSIZE		; test for the end of the data buffer
	brlo	Rs232Gbx
	clr	rxhead			; Circle around to the first buffer pos.
Rs232Gbx:
	std	Y+prxhead-RAMSTRT,rxhead

		;***** Xon calcs go here *****
	sec				; This is not need for ASCII data
	ret				; as a zero value returned in tempchar works
Rs232GetBytexit:			; But, for binary data, zero may be valid!
	clc				; Return 'No' bytes in yet.
	ret


;******************************
;* And The RS232 GetChar Loop *
;* wait until a char comes in *
;******************************
GetChar:
	rcall	Rs232GetByte
	brcc	GetChar
	ret
;




;**********************************************
;* UART TX COMPLET 
;**********************************************
UART_TXC:	PUSH	ZL
		PUSH	ZH
		LDD	ZL,Y+UTxAddr+0-RAMSTRT
		LDD	ZH,Y+UTxAddr+1-RAMSTRT
		IJMP			;JUMP TO (Z) INTERRUPT RUTIN


;*****************************************************************************
;* Send TXT
;*****************************************************************************
SendTXT:	IN	ZL,SREG
		PUSH	ZL
		PUSH	R16
		PUSH	R0

		LDD	ZL,Y+TXT+0-RAMSTRT
		LDD	ZH,Y+TXT+1-RAMSTRT
                LPM					;(Z) -> R0
		ADIW	ZL,1
		MOV	R16,R0
		STD	Y+TXT+0-RAMSTRT,ZL
		STD	Y+TXT+1-RAMSTRT,ZH
                CPI	R16,0
		BRNE	MoreTXT				;VAN MEG SZOVEG

		LDI	R16,low(UReti)
		STD	Y+UTxAddr+0-RAMSTRT,R16
		LDI	R16,high(UReti)
		STD	Y+UTxAddr+1-RAMSTRT,R16		;UART TX INT. ADDR
                CPSE	R16,R16				;AZERT, HOGY AKOVETKEZO SOR KIMARADJON
MoreTXT:	OUT	UDR,R16
                POP	R0
		POP	R16
		POP	ZL
		OUT	SREG,ZL
UReti:		POP	ZH
		POP	ZL
		RETI





;*******************************************************************************
;R0-ban levo byte kiirasa az uartra (HEX) 

TOUART:		LDI	R17,48
		LDI	R18,7
    		
Wt1:		SBIS	USR,UDRE	;a kovetkezo kimarad, ha a TX reg ures
		 RJMP	Wt1    		
		MOV	R16,R0
		SWAP	R16
		ANDI	R16,0x0F
		CPI	R16,10
		BRCS	Kicsi1
		ADD	R16,R18
Kicsi1:		ADD	R16,R17
		OUT	UDR,R16


Wt2:		SBIS	USR,UDRE	;a kovetkezo kimarad, ha a TX reg ures
		 RJMP	Wt2    		
		MOV	R16,R0
		ANDI	R16,0x0F
		CPI	R16,10
		BRCS	Kicsi0
		ADD	R16,R18
Kicsi0:		ADD	R16,R17
		OUT	UDR,R16
		ret


