;***********************************************************************
;*
;* Number		:PLC szimulator
;* File Name		:"PLC1.asm"
;* Title		:AVR alapu PLC 
;* 
;* Date	 		:2000.09.03.
;* Version 		:1.0
;* Support telephone	:+36-30-9541-658
;* Support fax		:+36-30-9809-372
;* Support E-mail	:vinczef@westel900.net
;* Target MCU		:AT90S8535
;*
;* DESCRIPTION
;* kezeli a kovetkezo eszkozoket:
;*  - RS232 fulduplex 
;*  - SPI interface
;*  - ADC stb..
;*
;*	AT90S8535 PDIP tok lbkioszts
;*
;*		 T0 (PB0) 1.	40. PA0 (ADC0)
;*	         T1 (PB1) 2.	39. PA1 (ADC1)
;*	       AIN0 (PB2) 3.	38. PA2 (ADC2)
;*	       AIN1 (PB3) 4.	37. PA3 (ADC3)
;*     VCC->8.2k SS (PB4) 5.	36. PA4 (ADC4)
;*           * MOSI (PB5) 6.	35. PA5 (ADC5)
;*	     * MISO (PB6) 7.	34. PA6 (ADC6)
;*	      * SCK (PB7) 8.	33. PA7 (ADC7)
;*		    RESET 9.	32. AREF     (Osszkotve az AVCC-vel)
;*		    VCC  10.	31. AGND 
;*      	    GND  11.	30. AVCC
;*     	           XTAL2 12.	29. PC7 (TOSC2)
;*     	           XTAL1 13.	28. PC6 (TOSC1)
;*	    *  RxD (PD0) 14.	27. PC5
;*	    *  TxD (PD1) 15.	26. PC4
;*	      INT0 (PD2) 16.	25. PC3
;*	      INT1 (PD3) 17.	24. PC2
;*	      OC1B (PD4) 18.	23. PC1 
;*            OC1A (PD5) 19.	22. PC0
;*     	       ICP (PD6) 20.	21. (PD7) OC2
;*
;*	Kristlyfrekvencia: f=3.6864 MHz (T=0.27126 us)
;*
;*	* a csillagos IO labak fix huzalozasuak, masra nem hasznalhatok fel!!!
;***********************************************************************
.include "8535def.inc"


;**************************************************************************
;* Const Def
.EQU	BaudRate=23	;23 - 9600 @3.6864MHz

.EQU	T0Freq=184	;220 @1.8432MHz = 50Hz
                        ;184 @3.6864MHz = 50Hz



.EQU	ADCoef	= 20





;***************************************************************************
;***** Divide Subroutine Register Variables

.def	drem16uL=r14
.def	drem16uH=r15
.def	dres16uL=r16
.def	dres16uH=r17
.def	dd16uL	=r16
.def	dd16uH	=r17
.def	dv16uL	=r18
.def	dv16uH	=r19


;**************************************************************************
;Multiplier Registers

.def	uuMANT1l=r23	; ACC1 LSByte
.def	uuMANT1h=r22
.def	uuMANT2l=r21	; ACC2
.def	uuMANT2h=r20
.def	uuPRODl	=r19	; 16bit Multiplier
.def	uuPRODh	=r18
.def	uucount	=r17	; bit count
.def	uutemp	=r16



;******************************************************************************
;**** VARIABLES
.DSEG
Flag1:	 .BYTE 1		;Flagbitek

UTxAddr: .BYTE 2		;UART Transmit Interrupt rutin cime
prxhead: .BYTE 1		;RS232 rxBuff Head
prxtail: .BYTE 1		;RS232 rxBuff Tail

TXT:     .BYTE 2		;RS232 Send ROM Text Addr
ptxhead: .BYTE 1		;RS232 txBuff Head
ptxtail: .BYTE 1		;RS232 txBuff Tail


ADC_CH5: .BYTE 2		;Line status Voltage Last Value
ADC_CH4: .BYTE 2		;Current Monitor Last Value

RTC:     .BYTE 4		;RTC 0=1970/01/01 00:00 bekapcsolasi ertek


;**************************************************************************************
.ESEG
STPWM:	.DB	0x80		;PWM Init value low
	.DB	0x00		;PWM init high


;*****************************************************************************
;****		I N T E R R U P T S
;****
;*****************************************************************************
.CSEG
		rjmp RESET	 ; Reset Handler
		rjmp EXT_INT0	 ; IRQ0 Handler
		rjmp EXT_INT1	 ; IRQ1 Handler
		rjmp TIM2_COMP	 ; Timer2 Compare Handler
		rjmp TIM2_OVF	 ; Timer2 Overflow Handler
		rjmp TIM1_CAPT	 ; Timer1 Capture Handler
		rjmp TIM1_COMPA	 ; Timer1 CompareA Handler
		rjmp TIM1_COMPB	 ; Timer1 CompareB Handler
		rjmp TIM1_OVF	 ; Timer1 Overflow Handler
		rjmp TIM0_OVF	 ; Timer0 Overflow Handler
		rjmp SPI_INT	 ; SPI Transfer Complete Handler
		rjmp UART_RXC	 ; UART RX Complete Handler
		rjmp UART_DRE	 ; UDR Empty Handler
		rjmp UART_TXC	 ; UART TX Complete Handler
		rjmp ADC_INT	 ; ADC Conversion Complete Interrupt Handler
		rjmp EPR_INT	 ; EEPROM Ready Handler
		rjmp ACI_INT	 ; Analog Comparator Handler


;*******************
EXT_INT0:	RETI
EXT_INT1:	RETI
TIM2_COMP:	RETI
TIM2_OVF:	RETI
TIM1_CAPT:	RETI
TIM1_COMPA:	RETI
TIM1_COMPB:	RETI
TIM1_OVF:	RETI

;**********************************************
;* RealTime Clock Int
;*********************
;* 
TIM0_OVF:	PUSH	ZL
		PUSH	ZH
		IN	ZL,SREG
		PUSH	ZL

		LDI	ZL,T0Freq
		OUT	TCNT0,ZL	;50Hz

		LDD	ZL,Y+RTC-RAMSTRT
		LDD	ZH,Y+RTC+1-RAMSTRT
		ADIW	ZL,1
		STD	Y+RTC+0-RAMSTRT,ZL
		STD	Y+RTC+1-RAMSTRT,ZH
		BRCC	TIM_02

		LDD	ZL,Y+RTC+2-RAMSTRT
		LDD	ZH,Y+RTC+3-RAMSTRT
		ADIW	ZL,1
		STD	Y+RTC+2-RAMSTRT,ZL
		STD	Y+RTC+3-RAMSTRT,ZH

TIM_02:		POP	ZL
		OUT	SREG,ZL
		POP	ZH
		POP	ZL
		RETI



;**************************************************
;* SPI Int Handle
;********************
SPI_INT:	RETI 


EPR_INT:	RETI
ACI_INT:	RETI




.include	"RS232.asm"
.include	"div16.asm"
.include	"mult16.asm"
.include	"ADC8535.asm"






;**********************************************************************
;* RESET 
;**********************************************************************
RESET:	CLI			;GLOBAL INTERRUP DIS.	

	ldi r16, high(RAMEND)	;setup stack
	out SPH,r16
	ldi r16, low(RAMEND)
	out SPL,r16

	LDI	R16,42		;Sleep enable, Idle mode,INT0, INT1 fall-edge sens.
	OUT	MCUCR,R16


	LDI	R16,0		;INT0, INT1 Disabled
	OUT	GIMSK,R16	;7. = 1 EXT INT1 ACTIVE
				;6. = 1 EXT INT0 ACTIVE

	LDI	R16,2
	OUT	TIMSK,R16 	;7. = 1 TC1 OWRF EN.  *
				;6. = 1 TC1 COMP MATCH EN.
				;3. = 1 TC1 INP CAPTURE EN.
				;1. = 1 TC0 OWRF EN.   *
	

	LDI	YL,LOW(RAMSTRT)
	LDI	YH,HIGH(RAMSTRT) ;BAZIS 'Y' +0

	LDI	R16,0


	STD	Y+RTC+0-RAMSTRT,R16	;RealTime Clock=1970\01\01
	STD	Y+RTC+1-RAMSTRT,R16
	STD	Y+RTC+2-RAMSTRT,R16
	STD	Y+RTC+3-RAMSTRT,R16

	OUT	TCNT1H,R16
	OUT	TCNT1L,R16	;=0x0000


;	ldi	R16,STPWM	;PWM init value in EEPROM
;	out	EEAR,R16	;output address 8515
;	rcall	EERead_seq	;PWM LOW in R0
;	mov	R20,R0
;	rcall	EERead_seq	;PWM hi in R0
;	OUT	OCR1H,R0
;	OUT	OCR1L,R20	;PWM=int value



;	LDI	R17,128+1
;	OUT	TCCR1A,R17	;Non-invert 8bit PWM, 
;	LDI	R17,1
;	OUT	TCCR1B,R17	;Input Capture Dis., f=CLK


;	SBI	DDRB,3		;PWM kimenet
	


;	*****************************
;	**  Timer0 Init		*****

	LDI	R16,T0Freq
	OUT	TCNT0,R16	;50Hz
	LDI	R16,5
	OUT	TCCR0,R16	;f(TC1)=CLK/1024


	rcall	Init_UART


	SEI			;SET GLOBAL INT EN.

;	rcall	Init_ADC


	LDI	R16,13
	OUT	UDR,R16		;INITSTRING	


;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:	


wait:		;SLEEP


		rcall	Rs232GetByte
		brcc	Csokkent		
		CBR	R18,32		;kisbetu -> NAGYBETU konverzio csak betuk!!

		CPI	R18,'R'		;Read ADC
		BREQ	ADCR5ch


		CPI	R18,'C'		;Read ADC current
		BREQ	ADCR4ch


		rjmp	Csokkent


ADCR4ch:	ldd	R0,Y+ADC_CH4+1-RAMSTRT

		rcall	TOUART
		mov	R0,uuMANT1l
		
		ldd	R0,Y+ADC_CH4+0-RAMSTRT

		rcall	TOUART
		rjmp	Wt6


ADCR5ch:	mov	uuMANT1l,R0		;ADC value
		ldi	uuMANT1h,0

		ldi	uuMANT2l,low(ADCoef)	;ADC coef
		ldi	uuMANT2h,high(ADCoef)


		rcall	UMUL16			;Bin to mV converter


		mov	R0,uuMANT1h

		ldd	R0,Y+ADC_CH5+1-RAMSTRT


		rcall	TOUART
		mov	R0,uuMANT1l
		
		ldd	R0,Y+ADC_CH5+0-RAMSTRT

		rcall	TOUART


Wt6:		SBIS	USR,UDRE	;a kovetkezo kimarad, ha a TX reg ures
		 RJMP	Wt6    		
		LDI	R16,' '
		OUT	UDR,R16	



CMDsEND:	SBIS	USR,UDRE	;a kovetkezo kimarad, ha a TX reg ures
		 RJMP	CMDsEND    		
		LDI	R16,13
		OUT	UDR,R16	

Wt9:		SBIS	USR,UDRE	;a kovetkezo kimarad, ha a TX reg ures
		 RJMP	Wt9    		
		LDI	R16,10
		OUT	UDR,R16	

CLRCMD:		nop


Csokkent:

		RJMP	Main









;***************************************************************************
;* 
;* EERead_seq
;*
;* This subroutine increments the address stored in EEAR and reads the 
;* EEPROM into the register variable "EEdrd_s".
;*
;* Number of words	:8515 ; 7 + return
;* Number of cycles	:8515 ; 11 + return (if EEPROM is ready)
;* Low Registers used	:R0
;* High Registers used:	:R24
;*
;***************************************************************************
EERead_seq:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EERead_seq	;   wait more
				;The above sequence for EEWE = 0
				;can be skipped if no write is initiated.
	sbi	EECR,EERE	;set EEPROM Read strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	R0,EEDR		;get data

	in	R24,EEARL	;get address low 8535
	inc	R24		;increment address 8535
	out	EEARL,R24	;output address 8535
	brcc	EEok
	in	R24,EEARH	;get address low 8535
	inc	R24		;increment address 8535
	out	EEARH,R24	;output address 8535
     	

EEok:	ret





